var vows = require( 'vows' ),
    assert = require( 'assert' ),
    equality = require( '../lib/equality' );

vows.describe( 'Equality' ).addBatch({

  'deep()': {

    topic: function () { return equality.deep; },

    'works for atomic values': function ( deep ) {
      // numbers
      assert.isTrue( deep( 1, 1 ) );
      assert.isFalse( deep( 1, 2 ) );

      // numbers and objects
      assert.isFalse( deep( 1, { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, 1 ) );

      // numbers and booleans
      assert.isFalse( deep( 1, true ) );
      assert.isFalse( deep( true, 1 ) );

      // numbers and strings
      assert.isFalse( deep( 1, 'one' ) );
      assert.isFalse( deep( 'one', 1 ) );

      // booleans
      assert.isTrue( deep( true, true ) );
      assert.isFalse( deep( true, false ) );

      // booleans and objects
      assert.isFalse( deep( true, { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, true ) );

      // booleans and strings
      assert.isFalse( deep( true, 'true' ) );
      assert.isFalse( deep( 'true', true ) );

      // strings
      assert.isTrue( deep( 'string', 'string' ) );
      assert.isFalse( deep( 'string', 'number' ) );

      // strings and objects
      assert.isFalse( deep( 'string', { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, 'string' ) );
    },

    'works for objects with objects': function ( deep ) {
      // deep
      var A = { a: { b: 'c' }, d: { e: 'f' } },
          B = { a: { b: 'c' }, d: { e: 'f' } },
          C = { x: 10, y: { e: 'f' } };

      assert.isTrue( deep( A, B ) );
      assert.isFalse( deep( B, C ) );
      assert.isTrue( deep( A.d, C.y ) );

      // deeper!
      var X = { a: A, b: B, c: C },
          Y = { a: A, b: B, c: C },
          Z = { a: C, b: A, c: B };

      assert.isTrue( deep( X, Y ) );
      assert.isFalse( deep( Y, Z ) );
      assert.isTrue( deep( X.a, Z.b ) );

      // wtf is going on...
      var U = { x: X, y: Y, z: Z },
          V = { x: X, y: Y, z: Z },
          W = { x: Z, y: X, z: Y };

      //assert.isTrue( deep( U, V ) );
      //assert.isFalse( deep( V, W ) );
      //assert.isTrue( deep( V.x, W.y ) );
    }

  }

}).export( module );
